/**
 * Created by brooklyn on 14-2-21.
 */
var vee = vee || {};
vee.Analytics = {
    pAnalytics: null,
    orderInfo : null,

    loadPlugins: function(){

        this.unloadPlugin();
        this.pAnalytics = new Array();
        for (var i = 0; i < app.Config.AnalyticsPluginName.length; i++) {
            this.pAnalytics[i] = plugin.PluginManager.getInstance().loadPlugin(app.Config.AnalyticsPluginName[i]);
            if (!this.pAnalytics[i])
                continue;

            this.pAnalytics[i].setDebugMode(vee.Common.getInstance().isDebugMode());
            this.pAnalytics[i].setCaptureUncaughtException(true);
            //this.pAnalytics[i].startSession(app.Config.AnalysticsPluginConfig[i]);
        }
    },

    unloadPlugin: function(){
        if (this.pAnalytics) {
            for (var i = 0; i < this.pAnalytics.length; i++) {
                if (!this.pAnalytics[i])
                    continue;
                plugin.PluginManager.getInstance().unloadPlugin(this.pAnalytics[i].getPluginName());
                this.pAnalytics[i] = null;
            }
        }
        this.pAnalytics = null;
    },

    startSession: function(){
        if (this.pAnalytics){
            for (var i = 0; i < this.pAnalytics.length; i++) {
                if (!this.pAnalytics[i])
                    continue;
                this.pAnalytics[i].startSession(app.Config.AnalysticsPluginConfig[i]);
            }
        }
    },

    stopSession: function(){
        if (this.pAnalytics){
            for (var i = 0; i < this.pAnalytics.length; i++) {
                if (!this.pAnalytics[i])
                    continue;
                this.pAnalytics[i].stopSession();
            }
        }
    },

    logTimedEventBegin: function(eventID){
        if (this.pAnalytics){
            for (var i = 0; i < this.pAnalytics.length; i++) {
                if (!this.pAnalytics[i])
                    continue;
                this.pAnalytics[i].logTimedEventBegin(eventID);
            }
        }
    },

    logTimedEventEnd: function(eventID){
        if (this.pAnalytics){
            for (var i = 0; i < this.pAnalytics.length; i++) {
                if (!this.pAnalytics[i])
                    continue;
                this.pAnalytics[i].logTimedEventEnd(eventID);
            }
        }
    },

    logEvent: function(eventID, paramMap){
        //if (eventID != "RechargeSuccess")
        //    return;

        if (this.pAnalytics){
            for (var i = 0; i < this.pAnalytics.length; i++) {
                if (!this.pAnalytics[i])
                    continue;

                if (!paramMap)  {
                    paramMap = {"CIS":"Test",};
                    //vee.alert("TestLogEvent", "Empty paraMap");
                }

                this.pAnalytics[i].logEvent(eventID, paramMap);
            }
        }
    },

    /**
     * Log player level.
     * @param {number} level
     */
    logPlayerLevel: function(level){
        var para = { "Level" : ""+level, };
        this.logEvent("LevelUp", para);  

        /*
        if (this.pAnalytics)
        {
            this.pAnalytics.callFuncWithParam("logPlayerLevel",
                new plugin.PluginParam(plugin.PluginParam.ParamType.TypeInt, level)
            );
        }
        */
    },
    /**
     * Log purchase virtual product with virtual currency.
     * @param {string} productId
     * @param {number} number
     * @param {number} price
     */
    logItemPurchase: function(productId, price, number){
        var para = {
            "ItemID" : ""+productId,
            "Price" : ""+price,
            "Count" : ""+number,
        };

        this.logEvent("ItemPurchase", para);

        /*
        if (this.pAnalytics){
            number = number || 1;
            this.pAnalytics.callFuncWithParam("logItemPurchase",
                new plugin.PluginParam(plugin.PluginParam.ParamType.TypeStringMap, {
                    "itemID": productId,
                    "number": number.toString(),
                    "price" : price.toString()
                })
            );
        }
        */
    },
    /**
     * Log when user use product.
     * @param {string} productId
     * @param {number} number
     */
    logItemUse: function(productId, number, price){
        var para = {
            "ItemID" : ""+productId,
            "Price" : ""+price,
            "Count" : ""+number,
        };

        this.logEvent("ItemUse", para);

        /*
        if (this.pAnalytics){
            number = number || 1;
            if ("AnalyticsTalkingGame" == app.Config.AnalyticsPluginName) {
                this.pAnalytics.callFuncWithParam("logItemUse",
                    new plugin.PluginParam(plugin.PluginParam.ParamType.TypeStringMap, {
                        "itemID": productId,
                        "number": number.toString()
                    })
                );
            }
            if ("AnalyticsUmengGame" == app.Config.AnalyticsPluginName) {
                this.pAnalytics.callFuncWithParam("logItemUse",
                    new plugin.PluginParam(plugin.PluginParam.ParamType.TypeStringMap, {
                        "itemID": productId,
                        "number": number.toString(),
                        "price" : price.toString()
                    })
                );
            }
        }
        */
    },


    /**
     * Log IAP.
     * @param {string} orderId
     * @param {string} iapID
     * @param {number} moneyAmount
     * @param {string} moneyType
     * @param {number} productAmount
     * @param {string} paymentType
     */
    logChargeRequest: function(orderId, iapID, moneyAmount, moneyType, productAmount, paymentType) {
        var para = {
            "OrderID" : orderId,
            "ItemID" : iapID,
            "Price" : ""+moneyAmount,
            "MoneyType" : moneyType,
            "Value" : ""+productAmount,
            "PaymentType" : paymentType,
        };

        this.orderInfo = para;

        this.logEvent("BeforeRecharge", para);

        /*
        if (this.pAnalytics) {
            moneyType = moneyType || "USD";
            productAmount = productAmount || 1;
            if (!paymentType) {
                if (cc.sys.os === cc.sys.OS_ANDROID) {
                    paymentType = "Google";
                } else if (cc.sys.os === cc.sys.OS_IOS) {
                    paymentType = "Apple";
                } else {
                    paymentType = "Unknown";
                }
            }
            if (_.isNumber(iapID)) {
                iapID = app.Config.IAPs[iapID].ProductID;
            }
            if ("AnalyticsTalkingGame" == app.Config.AnalyticsPluginName) {
                this.pAnalytics.callFuncWithParam("logChargeRequest",
                    new plugin.PluginParam(plugin.PluginParam.ParamType.TypeStringMap, {
                        "orderID": orderId,
                        "iapID": iapID,
                        "currencyAmount": moneyAmount.toString(),
                        "currencyType": moneyType,
                        "productAmount": productAmount.toString(),
                        "paymentType": paymentType
                    })
                );
            }
            if ("AnalyticsUmengGame" == app.Config.AnalyticsPluginName) {
                this.orderInfo = {
                    "iapID": iapID,
                    "currencyAmount": moneyAmount.toString(),
                    "currencyType": moneyType,
                    "productAmount": productAmount.toString(),
                    "paymentType": paymentType
                }
            }
        }
         */
    },


    /**
     * Log IAP Success.
     * @param {string} orderId
     */
    logChargeSuccess: function(orderId){
        if (this.orderInfo == null) {
            vee.alert("ChargeSuccess", "Charge success without orderInfo.");
            return;
        }

        this.logEvent("RechargeSuccess", this.orderInfo);

        /*
        if (this.pAnalytics){
            if ("AnalyticsTalkingGame" == app.Config.AnalyticsPluginName) {
                this.pAnalytics.callFuncWithParam("logChargeSuccess",
                    new plugin.PluginParam(plugin.PluginParam.ParamType.TypeString, orderId)
                );
            }
            if ("AnalyticsUmengGame" == app.Config.AnalyticsPluginName) {
                this.pAnalytics.callFuncWithParam("logChargeSuccess",
                    new plugin.PluginParam(plugin.PluginParam.ParamType.TypeStringMap, this.orderInfo)
                );
            }
        }
        */
    },

    logMissionStart : function(missionId){
        var para = {
            "MissionID" : ""+missionId,
        };

        this.logEvent("MissionStart", para);

        /*
        if (this.pAnalytics){
            this.pAnalytics.callFuncWithParam("logMissionStart",
                new plugin.PluginParam(plugin.PluginParam.ParamType.TypeString, missionId)
            );
        }
        */
    },

    logMissionCompleted : function(missionId){
        var para = {
            "MissionID" : ""+missionId,
        };

        this.logEvent("MissionCompleted", para);
        /*
        if (this.pAnalytics){
            this.pAnalytics.callFuncWithParam("logMissionCompleted",
                new plugin.PluginParam(plugin.PluginParam.ParamType.TypeString, missionId)
            );
        }
        */
    },

    logMissionFailed : function(missionId, cause){
        var para = {
            "MissionID" : ""+missionId,
        };

        this.logEvent("MissionFailed", para);

        /*
        if (this.pAnalytics){
            this.pAnalytics.callFuncWithParam("logMissionFailed",
                new plugin.PluginParam(plugin.PluginParam.ParamType.TypeStringMap, {
                    "missionId": missionId,
                    "cause": cause
                })
            );
        }
        */
    },


    //UGame extra reward
    /*  @param {int} type   0代表赠送虚拟币，使用coin + eventID参数,1代表赠送道具，使用item,num,price,eventID参数
     *   @param {string} item
     *   @param {number} num
     *   @param {number} price
     *   @param {number} eventID
     *   @param {string} paymentType
     */
    logBonus : function(paramMap) {
        this.logEvent("Bonus", paramMap);

        /*
         this.pAnalytics.callFuncWithParam("logBonus",
         new plugin.PluginParam(plugin.PluginParam.ParamType.TypeStringMap, paramMap)
         );
         */
    },

    /*
    logReward : function(virtualCurrencyAmount, reason) {
        if (this.pAnalytics){
            this.pAnalytics.callFuncWithParam("logReward",
                new plugin.PluginParam(plugin.PluginParam.ParamType.TypeStringMap, {
                    "virtualCurrencyAmount": ''+virtualCurrencyAmount,
                    "reason": reason
                })
            );
        }
    },
    */


    /*
    logAdTrackingCustEvent1 : function(){
        if (vee.tracking){
            vee.tracking.callFuncWithParam("logAdTrackingCustEvent1");
        }
    },

    logAdTrackingCustEvent2 : function(){
        if (vee.tracking){
            vee.tracking.callFuncWithParam("logAdTrackingCustEvent2");
        }
    },

    logAdTrackingCustEvent3 : function(){
        if (vee.tracking){
            vee.tracking.callFuncWithParam("logAdTrackingCustEvent3");
        }
    },

    logAdTrackingRegister : function(account){
        if (vee.tracking){
            vee.tracking.callFuncWithParam("logAdTrackingRegister",
                new plugin.PluginParam(plugin.PluginParam.ParamType.TypeString, account)
            );
        }
    },

    logAdTrackingLogin : function(account){
        if (vee.tracking){
            vee.tracking.callFuncWithParam("logAdTrackingLogin",
                new plugin.PluginParam(plugin.PluginParam.ParamType.TypeString,  account)
            );
        }
    }
    */
};

vee.Analytics.activate = function(){
    this.loadPlugins();
};

/*
vee.Analytics.getAnalytics = function(pluginName, pluginConfig){
    var p = plugin.PluginManager.getInstance().loadPlugin(pluginName);
    if (!p) return null;
    p.setDebugMode(vee.Common.getInstance().isDebugMode());
    p.startSession(pluginConfig);
    p.setCaptureUncaughtException(true);
    return p;
};
*/

// Ad Tracking...
vee.AnalyticsAdTracking = {
	pAnalytics: null,

	loadPlugins: function(){
		/*
        this.unloadPlugin();
		this.pAnalytics = plugin.PluginManager.getInstance().loadPlugin(app.Config.AnalyticsPluginADTrackName);
		if (!this.pAnalytics) return;
		this.pAnalytics.setDebugMode(vee.Common.getInstance().isDebugMode());
		this.pAnalytics.startSession(app.Config.AnalyticsPluginADTrackConfig);
		this.pAnalytics.setCaptureUncaughtException(true);
		*/
	},

	unloadPlugin: function(){
		if (this.pAnalytics){
			plugin.PluginManager.getInstance().unloadPlugin(this.pAnalytics.getPluginName());
			this.pAnalytics = null;
		}
	},

	logEvent: function(eventID, paramMap){
		if (this.pAnalytics){
			if (arguments.length > 1){
				this.pAnalytics.logEvent(eventID, paramMap)
			} else {
				this.pAnalytics.logEvent(eventID);
			}
		}
	}
};

vee.AnalyticsAdTracking.activate = function() {
	this.loadPlugins();
	this.logEvent("Init plugin", app.Config.AnalyticsPluginADTrackConfig);
}